def run(cmdName, args):
	from Midas.midas_text import doExtensionFunc
	doExtensionFunc(mapsums, args, specInfo=[
					("atom_spec", "atoms", "atoms"),
					("map_spec", "maps", "models"), ])

def mapsums(maps, atoms, radiiList="3.0,5.0,7.0",
				gridSpacing=0.3, filename=None):
	# Report densities for open molecule and map.
	try:
		radii = [ float(v) for v in radiiList.split(',') ]
	except ValueError:
		from chimera import UserError
		raise UserError("radii list is not a comma-separated "
				"list of floating point numbers")
	if not maps:
		from chimera import UserError
		raise UserError("no volume selected")
	if not atoms:
		from chimera import UserError
		raise UserError("no atoms selected")
	grids = [ grid_points(r, gridSpacing) for r in radii ]
	for g in grids:
		print len(g)
		print g
	results = dict()
	for map in maps:
		results[map] = [ atom_densities(atoms, g, map) for g in grids ]
	gridSize = [ len(g) for g in grids ]
	if filename is None:
		import chimera
		if not chimera.nogui:
			_showResults(results, maps, atoms,
						radii, gridSize, gridSpacing)
		else:
			import sys
			_printResults(sys.stdout, results, maps, atoms,
						radii, gridSize, gridSpacing)
			chimera.dialogs.display("reply")
	else:
		with file(filename, "w") as f:
			_printResults(f, results, maps, atoms,
						radii, gridSize, gridSpacing)

def _showResults(results, maps, atoms, radii, gridSize, gridSpacing):
	import sys
	_printResults(sys.stdout, results, maps, atoms,
						radii, gridSize, gridSpacing)
	from gui import MapSumsResultsDialog
	for map in maps:
		MapSumsResultsDialog(results[map], map, atoms,
						radii, gridSize, gridSpacing)

def _printResults(f, results, maps, atoms, radii, gridSize, gridSpacing):
	for map in maps:
		printMapResults(f, results[map], map.name, atoms, radii,
						gridSize, gridSpacing)

def printMapResults(f, aves, mapName, atoms, radii, gridSize, gridSpacing):
	rtext = ' '.join([ "%.4g" % r for r in radii ])
	ngtext = ' '. join([ "%d" % g for g in gridSize ])
	print >> f, "average density near %d atoms in map %s, " \
					% (len(atoms), mapName)
	print >> f, "radii %s, sampling grid spacing %.4g, " \
					"grid points %s" \
					% (rtext, gridSpacing, ngtext)
	for i, a in enumerate(atoms):
		if a is None:
			atomName = "-"
			position = -1
			chainId = '-'
		else:
			atomName = a.name
			rid = a.residue.id
			position = rid.position
			chainId = rid.chainId
		atext = "   ".join([ "%8.3g" % rAve[i]
					for rAve in aves ])
		print >> f, "%4d %4s %4s %s" % (position, chainId,
						atomName, atext)

# Average density around atoms.

def atom_densities(atoms, sampling_grid, map):

	centers = atom_coordinates(atoms, map.openState.xform)
	aves = average_density(centers, sampling_grid, map)
	return aves

# Compute atom coordinates in map coordinate system
def atom_coordinates(atoms, map_xf):

	xf = map_xf.inverse()
	axyz = [ xf.apply(a.xformCoord()).data() for a in atoms ]
	return axyz

# Compute average density inside sphere using grid centered on sphere.
def average_density(centers, sampling_grid, map):

	n = len(sampling_grid)
	aves = []
	for c in centers:
		v = map.interpolated_values(sampling_grid + c)
		aves.append(float(v.sum()) / n)
	return aves

# Produce array of grid points inside sphere centered at 0,0,0.
def grid_points(radius, grid_spacing):

	g = []
	n = int(float(radius + grid_spacing) / grid_spacing)
	r2 = radius*radius
	for i in range(-n, n + 1):
		for j in range(-n, n + 1):
			for k in range(-n, n + 1):
				dx = i*grid_spacing
				dy = j*grid_spacing
				dz = k*grid_spacing
				if dx * dx + dy * dy + dz * dz < r2:
					g.append((dx, dy, dz))
	import numpy
	ga = numpy.array(g)
	return ga
